(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
	typeof define === 'function' && define.amd ? define(factory) :
	(global.vueDirectiveTooltip = factory());
}(this, (function () { 'use strict';

var commonjsGlobal = typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};





function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var popper = createCommonjsModule(function (module, exports) {
var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**!
 * @fileOverview Kickass library to create and place poppers near their reference elements.
 * @version 1.5.1
 * @license
 * Copyright (c) 2016 Federico Zivolo and contributors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
(function (global, factory) {
  (_typeof(exports)) === 'object' && 'object' !== 'undefined' ? module.exports = factory() : typeof undefined === 'function' && undefined.amd ? undefined(factory) : global.Popper = factory();
})(commonjsGlobal, function () {
  'use strict';

  /**
   * Get CSS computed property of the given element
   * @method
   * @memberof Popper.Utils
   * @argument {Eement} element
   * @argument {String} property
   */

  function getStyleComputedProperty(element, property) {
    if (element.nodeType !== 1) {
      return [];
    }
    // NOTE: 1 DOM access here
    var css = window.getComputedStyle(element, null);
    return property ? css[property] : css;
  }

  /**
   * Returns the parentNode or the host of the element
   * @method
   * @memberof Popper.Utils
   * @argument {Element} element
   * @returns {Element} parent
   */
  function getParentNode(element) {
    if (element.nodeName === 'HTML') {
      return element;
    }
    return element.parentNode || element.host;
  }

  /**
   * Returns the scrolling parent of the given element
   * @method
   * @memberof Popper.Utils
   * @argument {Element} element
   * @returns {Element} scroll parent
   */
  function getScrollParent(element) {
    // Return body, `getScroll` will take care to get the correct `scrollTop` from it
    if (!element || ['HTML', 'BODY', '#document'].indexOf(element.nodeName) !== -1) {
      return window.document.body;
    }

    // Firefox want us to check `-x` and `-y` variations as well

    var _getStyleComputedProp = getStyleComputedProperty(element),
        overflow = _getStyleComputedProp.overflow,
        overflowX = _getStyleComputedProp.overflowX,
        overflowY = _getStyleComputedProp.overflowY;

    if (/(auto|scroll)/.test(overflow + overflowY + overflowX)) {
      return element;
    }

    return getScrollParent(getParentNode(element));
  }

  function isOffsetContainer(element) {
    return element.nodeName === 'HTML' || element.firstElementChild.offsetParent === element;
  }

  /**
   * Finds the root node (document, shadowDOM root) of the given element
   * @method
   * @memberof Popper.Utils
   * @argument {Element} node
   * @returns {Element} root node
   */
  function getRoot(node) {
    if (node.parentNode !== null) {
      return getRoot(node.parentNode);
    }

    return node;
  }

  /**
   * Finds the offset parent common to the two provided nodes
   * @method
   * @memberof Popper.Utils
   * @argument {Element} element1
   * @argument {Element} element2
   * @returns {Element} common offset parent
   */
  function findCommonOffsetParent(element1, element2) {
    var range = document.createRange();
    // This check is needed to avoid errors in case one of the elements isn't defined for any reason
    if (element1 && element2) {
      // Here we make sure to give as "start" the element that comes first in the DOM
      if (element1.compareDocumentPosition(element2) & Node.DOCUMENT_POSITION_FOLLOWING) {
        range.setStart(element1, 0);
        range.setEnd(element2, 0);
      } else {
        range.setStart(element2, 0);
        range.setEnd(element1, 0);
      }
    } else {
      return window.document.documentElement;
    }

    var commonAncestorContainer = range.commonAncestorContainer;

    // Both nodes are inside #document

    if (element1 !== commonAncestorContainer && element2 !== commonAncestorContainer) {
      if (isOffsetContainer(commonAncestorContainer)) {
        return commonAncestorContainer;
      }

      var offsetParent = commonAncestorContainer && commonAncestorContainer.offsetParent;

      if (!offsetParent || offsetParent && offsetParent.nodeName === 'BODY') {
        return window.document.documentElement;
      }

      return offsetParent;
    }

    // one of the nodes is inside shadowDOM, find which one
    var element1root = getRoot(element1);
    if (element1root.host) {
      return findCommonOffsetParent(element1root.host, element2);
    } else {
      return findCommonOffsetParent(element1, getRoot(element2).host);
    }
  }

  /**
   * Gets the scroll value of the given element in the given side (top and left)
   * @method
   * @memberof Popper.Utils
   * @argument {Element} element
   * @argument {String} side `top` or `left`
   * @returns {Number} amount of scrolled pixels
   */
  function getScroll(element) {
    var side = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'top';

    var upperSide = side === 'top' ? 'scrollTop' : 'scrollLeft';
    var nodeName = element.nodeName;

    if (nodeName === 'BODY' || nodeName === 'HTML') {
      var html = window.document.documentElement;
      var scrollingElement = window.document.scrollingElement || html;
      return scrollingElement[upperSide];
    }

    return element[upperSide];
  }

  /*
   * Sum or subtract the element scroll values (left and top) from a given rect object
   * @method
   * @memberof Popper.Utils
   * @param {Object} rect - Rect object you want to change
   * @param {HTMLElement} element - The element from the function reads the scroll values
   * @param {Boolean} subtract - set to true if you want to subtract the scroll values
   * @return {Object} rect - The modifier rect object
   */
  function includeScroll(rect, element) {
    var subtract = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

    var scrollTop = getScroll(element, 'top');
    var scrollLeft = getScroll(element, 'left');
    var modifier = subtract ? -1 : 1;
    rect.top += scrollTop * modifier;
    rect.bottom += scrollTop * modifier;
    rect.left += scrollLeft * modifier;
    rect.right += scrollLeft * modifier;
    return rect;
  }

  /*
   * Helper to detect borders of a given element
   * @method
   * @memberof Popper.Utils
   * @param {CSSStyleDeclaration} styles - result of `getStyleComputedProperty` on the given element
   * @param {String} axis - `x` or `y`
   * @return {Number} borders - the borders size of the given axis
   */

  function getBordersSize(styles, axis) {
    var sideA = axis === 'x' ? 'Left' : 'Top';
    var sideB = sideA === 'Left' ? 'Right' : 'Bottom';

    return Number(styles['border' + sideA + 'Width'].split('px')[0]) + Number(styles['border' + sideB + 'Width'].split('px')[0]);
  }

  function getWindowSizes() {
    var body = window.document.body;
    var html = window.document.documentElement;
    return {
      height: Math.max(body.scrollHeight, body.offsetHeight, html.clientHeight, html.scrollHeight, html.offsetHeight),
      width: Math.max(body.scrollWidth, body.offsetWidth, html.clientWidth, html.scrollWidth, html.offsetWidth)
    };
  }

  /**
   * Get bounding client rect of given element
   * @method
   * @memberof Popper.Utils
   * @param {HTMLElement} element
   * @return {Object} client rect
   */
  function getBoundingClientRect(element) {
    var isIE10 = navigator.appVersion.indexOf('MSIE 10') !== -1;
    var rect = void 0;

    // IE10 10 FIX: Please, don't ask, the element isn't
    // considered in DOM in some circumstances...
    // This isn't reproducible in IE10 compatibility mode of IE11
    if (isIE10) {
      try {
        rect = element.getBoundingClientRect();
      } catch (err) {
        rect = {};
      }
      var scrollTop = getScroll(element, 'top');
      var scrollLeft = getScroll(element, 'left');
      rect.top += scrollTop;
      rect.bottom += scrollTop;
      rect.left += scrollLeft;
      rect.right += scrollLeft;
    } else {
      rect = element.getBoundingClientRect();
    }

    var result = {
      left: rect.left,
      top: rect.top,
      right: rect.right,
      bottom: rect.bottom,
      width: rect.right - rect.left,
      height: rect.bottom - rect.top
    };

    // subtract scrollbar size from sizes
    var width = void 0,
        height = void 0;
    if (element.nodeName === 'HTML') {
      var sizes = getWindowSizes();
      width = sizes.width;
      height = sizes.height;
    } else {
      width = element.clientWidth || result.right - result.left;
      height = element.clientHeight || result.bottom - result.top;
    }
    var horizScrollbar = element.offsetWidth - width;
    var vertScrollbar = element.offsetHeight - height;

    // if an hypothetical scrollbar is detected, we must be sure it's not a `border`
    // we make this check conditional for performance reasons
    if (horizScrollbar || vertScrollbar) {
      var styles = getStyleComputedProperty(element);
      horizScrollbar -= getBordersSize(styles, 'x');
      vertScrollbar -= getBordersSize(styles, 'y');

      result.right -= horizScrollbar;
      result.width -= horizScrollbar;
      result.bottom -= vertScrollbar;
      result.height -= vertScrollbar;
    }

    return result;
  }

  function getOffsetRectRelativeToArbitraryNode(children, parent) {
    var isIE10 = navigator.appVersion.indexOf('MSIE 10') !== -1;
    var childrenRect = getBoundingClientRect(children);
    var parentRect = getBoundingClientRect(parent);
    var scrollParent = getScrollParent(children);
    var offsets = {
      top: childrenRect.top - parentRect.top,
      left: childrenRect.left - parentRect.left,
      bottom: childrenRect.top - parentRect.top + childrenRect.height,
      right: childrenRect.left - parentRect.left + childrenRect.width,
      width: childrenRect.width,
      height: childrenRect.height
    };

    // Subtract margins of documentElement in case it's being used as parent
    // we do this only on HTML because it's the only element that behaves
    // differently when margins are applied to it. The margins are included in
    // the box of the documentElement, in the other cases not.
    var isHTML = parent.nodeName === 'HTML';
    if (isHTML || parent.nodeName === 'BODY') {
      var styles = getStyleComputedProperty(parent);
      var borderTopWidth = isIE10 && isHTML ? 0 : +styles.borderTopWidth.split('px')[0];
      var borderLeftWidth = isIE10 && isHTML ? 0 : +styles.borderLeftWidth.split('px')[0];
      var marginTop = isIE10 && isHTML ? 0 : +styles.marginTop.split('px')[0];
      var marginLeft = isIE10 && isHTML ? 0 : +styles.marginLeft.split('px')[0];

      offsets.top -= borderTopWidth - marginTop;
      offsets.bottom -= borderTopWidth - marginTop;
      offsets.left -= borderLeftWidth - marginLeft;
      offsets.right -= borderLeftWidth - marginLeft;

      // Attach marginTop and marginLeft because in some circumstances we may need them
      offsets.marginTop = marginTop;
      offsets.marginLeft = marginLeft;
    }

    if (parent.contains(scrollParent) && (isIE10 || scrollParent.nodeName !== 'BODY')) {
      offsets = includeScroll(offsets, parent);
    }

    return offsets;
  }

  function getViewportOffsetRectRelativeToArtbitraryNode(element) {
    var html = window.document.documentElement;
    var relativeOffset = getOffsetRectRelativeToArbitraryNode(element, html);
    var width = Math.max(html.clientWidth, window.innerWidth || 0);
    var height = Math.max(html.clientHeight, window.innerHeight || 0);

    var scrollTop = getScroll(html);
    var scrollLeft = getScroll(html, 'left');

    return {
      top: scrollTop - relativeOffset.top + relativeOffset.marginTop,
      bottom: scrollTop - relativeOffset.top + relativeOffset.marginTop + height,
      left: scrollLeft - relativeOffset.left + relativeOffset.marginLeft,
      right: scrollLeft - relativeOffset.left + relativeOffset.marginLeft + width
    };
  }

  /**
   * Check if the given element is fixed or is inside a fixed parent
   * @method
   * @memberof Popper.Utils
   * @argument {Element} element
   * @argument {Element} customContainer
   * @returns {Boolean} answer to "isFixed?"
   */
  function isFixed(element) {
    var nodeName = element.nodeName;
    if (nodeName === 'BODY' || nodeName === 'HTML') {
      return false;
    }
    if (getStyleComputedProperty(element, 'position') === 'fixed') {
      return true;
    }
    return isFixed(getParentNode(element));
  }

  /**
   * Computed the boundaries limits and return them
   * @method
   * @memberof Popper.Utils
   * @param {Object} data - Object containing the property "offsets" generated by `_getOffsets`
   * @param {Number} padding - Boundaries padding
   * @param {Element} boundariesElement - Element used to define the boundaries
   * @returns {Object} Coordinates of the boundaries
   */
  function getBoundaries(popper, reference, padding, boundariesElement) {
    // NOTE: 1 DOM access here
    var boundaries = { top: 0, left: 0 };
    var offsetParent = findCommonOffsetParent(popper, reference);

    // Handle viewport case
    if (boundariesElement === 'viewport') {
      boundaries = getViewportOffsetRectRelativeToArtbitraryNode(offsetParent);
    } else {
      // Handle other cases based on DOM element used as boundaries
      var boundariesNode = void 0;
      if (boundariesElement === 'scrollParent') {
        boundariesNode = getScrollParent(getParentNode(popper));
        if (boundariesNode.nodeName === 'BODY') {
          boundariesNode = window.document.documentElement;
        }
      } else if (boundariesElement === 'window') {
        boundariesNode = window.document.documentElement;
      } else {
        boundariesNode = boundariesElement;
      }

      // In case of HTML, we need a different computation
      if (boundariesNode.nodeName === 'HTML' && !isFixed(offsetParent)) {
        var _getWindowSizes = getWindowSizes(),
            height = _getWindowSizes.height,
            width = _getWindowSizes.width;

        boundaries.right = width;
        boundaries.bottom = height;

        var offsets = getOffsetRectRelativeToArbitraryNode(boundariesNode, offsetParent);

        boundaries.top += offsets.top - offsets.marginTop;
        boundaries.bottom += offsets.top;
        boundaries.left += offsets.left - offsets.marginLeft;
        boundaries.right += offsets.left;
      } else {
        // for all the other DOM elements, this one is good
        boundaries = getOffsetRectRelativeToArbitraryNode(boundariesNode, offsetParent);
      }
    }

    // Add paddings
    boundaries.left += padding;
    boundaries.top += padding;
    boundaries.right -= padding;
    boundaries.bottom -= padding;

    return boundaries;
  }

  /**
   * Utility used to transform the `auto` placement to the placement with more
   * available space.
   * @method
   * @memberof Popper.Utils
   * @argument {Object} data - The data object generated by update method
   * @argument {Object} options - Modifiers configuration and options
   * @returns {Object} The data object, properly modified
   */
  function computeAutoPlacement(placement, refRect, popper, reference) {
    if (placement.indexOf('auto') === -1) {
      return placement;
    }

    var boundaries = getBoundaries(popper, reference, 0, 'scrollParent');

    var sides = {
      top: refRect.top - boundaries.top,
      right: boundaries.right - refRect.right,
      bottom: boundaries.bottom - refRect.bottom,
      left: refRect.left - boundaries.left
    };

    var computedPlacement = Object.keys(sides).sort(function (a, b) {
      return sides[b] - sides[a];
    })[0];
    var variation = placement.split('-')[1];

    return computedPlacement + (variation ? '-' + variation : '');
  }

  var nativeHints = ['native code', '[object MutationObserverConstructor]'];

  /**
   * Determine if a function is implemented natively (as opposed to a polyfill).
   * @argument {Function | undefined} fn the function to check
   * @returns {boolean}
   */
  var isNative = function isNative(fn) {
    return nativeHints.some(function (hint) {
      return (fn || '').toString().indexOf(hint) > -1;
    });
  };

  var isBrowser = typeof window !== 'undefined';
  var longerTimeoutBrowsers = ['Edge', 'Trident', 'Firefox'];
  var timeoutDuration = 0;
  for (var i = 0; i < longerTimeoutBrowsers.length; i += 1) {
    if (isBrowser && navigator.userAgent.indexOf(longerTimeoutBrowsers[i]) >= 0) {
      timeoutDuration = 1;
      break;
    }
  }

  function microtaskDebounce(fn) {
    var scheduled = false;
    var i = 0;
    var elem = document.createElement('span');

    // MutationObserver provides a mechanism for scheduling microtasks, which
    // are scheduled *before* the next task. This gives us a way to debounce
    // a function but ensure it's called *before* the next paint.
    var observer = new MutationObserver(function () {
      fn();
      scheduled = false;
    });

    observer.observe(elem, { attributes: true });

    return function () {
      if (!scheduled) {
        scheduled = true;
        elem.setAttribute('x-index', i);
        i = i + 1; // don't use compund (+=) because it doesn't get optimized in V8
      }
    };
  }

  function taskDebounce(fn) {
    var scheduled = false;
    return function () {
      if (!scheduled) {
        scheduled = true;
        setTimeout(function () {
          scheduled = false;
          fn();
        }, timeoutDuration);
      }
    };
  }

  // It's common for MutationObserver polyfills to be seen in the wild, however
  // these rely on Mutation Events which only occur when an element is connected
  // to the DOM. The algorithm used in this module does not use a connected element,
  // and so we must ensure that a *native* MutationObserver is available.
  var supportsNativeMutationObserver = isBrowser && isNative(window.MutationObserver);

  /**
  * Create a debounced version of a method, that's asynchronously deferred
  * but called in the minimum time possible.
  *
  * @method
  * @memberof Popper.Utils
  * @argument {Function} fn
  * @returns {Function}
  */
  var debounce = supportsNativeMutationObserver ? microtaskDebounce : taskDebounce;

  /**
   * Mimics the `find` method of Array
   * @method
   * @memberof Popper.Utils
   * @argument {Array} arr
   * @argument prop
   * @argument value
   * @returns index or -1
   */
  function find(arr, check) {
    // use native find if supported
    if (Array.prototype.find) {
      return arr.find(check);
    }

    // use `filter` to obtain the same behavior of `find`
    return arr.filter(check)[0];
  }

  /**
   * Return the index of the matching object
   * @method
   * @memberof Popper.Utils
   * @argument {Array} arr
   * @argument prop
   * @argument value
   * @returns index or -1
   */
  function findIndex(arr, prop, value) {
    // use native findIndex if supported
    if (Array.prototype.findIndex) {
      return arr.findIndex(function (cur) {
        return cur[prop] === value;
      });
    }

    // use `find` + `indexOf` if `findIndex` isn't supported
    var match = find(arr, function (obj) {
      return obj[prop] === value;
    });
    return arr.indexOf(match);
  }

  var _extends = Object.assign || function (target) {
    var arguments$1 = arguments;

    for (var i = 1; i < arguments.length; i++) {
      var source = arguments$1[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  /**
   * Given the popper offsets, generate an output similar to getBoundingClientRect
   * @method
   * @memberof Popper.Utils
   * @argument {Object} popperOffsets
   * @returns {Object} ClientRect like output
   */
  function getClientRect(popperOffsets) {
    return _extends({}, popperOffsets, {
      right: popperOffsets.left + popperOffsets.width,
      bottom: popperOffsets.top + popperOffsets.height
    });
  }

  /**
   * Returns the offset parent of the given element
   * @method
   * @memberof Popper.Utils
   * @argument {Element} element
   * @returns {Element} offset parent
   */
  function getOffsetParent(element) {
    // NOTE: 1 DOM access here
    var offsetParent = element.offsetParent;
    var nodeName = offsetParent && offsetParent.nodeName;

    if (!nodeName || nodeName === 'BODY' || nodeName === 'HTML') {
      return window.document.documentElement;
    }

    return offsetParent;
  }

  /**
   * Get the position of the given element, relative to its offset parent
   * @method
   * @memberof Popper.Utils
   * @param {Element} element
   * @return {Object} position - Coordinates of the element and its `scrollTop`
   */
  function getOffsetRect(element) {
    var elementRect = void 0;
    if (element.nodeName === 'HTML') {
      var _getWindowSizes2 = getWindowSizes(),
          width = _getWindowSizes2.width,
          height = _getWindowSizes2.height;

      elementRect = {
        width: width,
        height: height,
        left: 0,
        top: 0
      };
    } else {
      elementRect = {
        width: element.offsetWidth,
        height: element.offsetHeight,
        left: element.offsetLeft,
        top: element.offsetTop
      };
    }

    elementRect.right = elementRect.left + elementRect.width;
    elementRect.bottom = elementRect.top + elementRect.height;

    // position
    return elementRect;
  }

  /**
   * Get the outer sizes of the given element (offset size + margins)
   * @method
   * @memberof Popper.Utils
   * @argument {Element} element
   * @returns {Object} object containing width and height properties
   */
  function getOuterSizes(element) {
    var styles = window.getComputedStyle(element);
    var x = parseFloat(styles.marginTop) + parseFloat(styles.marginBottom);
    var y = parseFloat(styles.marginLeft) + parseFloat(styles.marginRight);
    var result = {
      width: element.offsetWidth + y,
      height: element.offsetHeight + x
    };
    return result;
  }

  /**
   * Get the opposite placement of the given one/
   * @method
   * @memberof Popper.Utils
   * @argument {String} placement
   * @returns {String} flipped placement
   */
  function getOppositePlacement(placement) {
    var hash = { left: 'right', right: 'left', bottom: 'top', top: 'bottom' };
    return placement.replace(/left|right|bottom|top/g, function (matched) {
      return hash[matched];
    });
  }

  /**
   * Get offsets to the popper
   * @method
   * @memberof Popper.Utils
   * @param {Object} position - CSS position the Popper will get applied
   * @param {HTMLElement} popper - the popper element
   * @param {Object} referenceOffsets - the reference offsets (the popper will be relative to this)
   * @param {String} placement - one of the valid placement options
   * @returns {Object} popperOffsets - An object containing the offsets which will be applied to the popper
   */
  function getPopperOffsets(position, popper, referenceOffsets, placement) {
    placement = placement.split('-')[0];

    // Get popper node sizes
    var popperRect = getOuterSizes(popper);

    // Add position, width and height to our offsets object
    var popperOffsets = {
      position: position,
      width: popperRect.width,
      height: popperRect.height
    };

    // depending by the popper placement we have to compute its offsets slightly differently
    var isHoriz = ['right', 'left'].indexOf(placement) !== -1;
    var mainSide = isHoriz ? 'top' : 'left';
    var secondarySide = isHoriz ? 'left' : 'top';
    var measurement = isHoriz ? 'height' : 'width';
    var secondaryMeasurement = !isHoriz ? 'height' : 'width';

    popperOffsets[mainSide] = referenceOffsets[mainSide] + referenceOffsets[measurement] / 2 - popperRect[measurement] / 2;
    if (placement === secondarySide) {
      popperOffsets[secondarySide] = referenceOffsets[secondarySide] - popperRect[secondaryMeasurement];
    } else {
      popperOffsets[secondarySide] = referenceOffsets[getOppositePlacement(secondarySide)];
    }

    return popperOffsets;
  }

  /**
   * Helper used to get the position which will be applied to the popper
   * @method
   * @memberof Popper.Utils
   * @param {HTMLElement} element - popper element
   * @returns {String} position
   */
  function getPosition(element) {
    var container = getOffsetParent(element);

    // Decide if the popper will be fixed
    // If the reference element is inside a fixed context, the popper will be fixed as well to allow them to scroll together
    var isParentFixed = isFixed(container);
    return isParentFixed ? 'fixed' : 'absolute';
  }

  /**
   * Get offsets to the reference element
   * @method
   * @memberof Popper.Utils
   * @param {Object} state
   * @param {Element} popper - the popper element
   * @param {Element} reference - the reference element (the popper will be relative to this)
   * @returns {Object} An object containing the offsets which will be applied to the popper
   */
  function getReferenceOffsets(state, popper, reference) {
    var commonOffsetParent = findCommonOffsetParent(popper, reference);
    return getOffsetRectRelativeToArbitraryNode(reference, commonOffsetParent);
  }

  /**
   * Get the prefixed supported property name
   * @method
   * @memberof Popper.Utils
   * @argument {String} property (camelCase)
   * @returns {String} prefixed property (camelCase)
   */
  function getSupportedPropertyName(property) {
    var prefixes = [false, 'ms', 'webkit', 'moz', 'o'];
    var upperProp = property.charAt(0).toUpperCase() + property.slice(1);

    for (var _i = 0; _i < prefixes.length - 1; _i++) {
      var prefix = prefixes[_i];
      var toCheck = prefix ? '' + prefix + upperProp : property;
      if (typeof window.document.body.style[toCheck] !== 'undefined') {
        return toCheck;
      }
    }
    return null;
  }

  /**
   * Gets the scroll value of the given element relative to the given parent/
   * It will not include the scroll values of elements that aren't positioned.
   * @method
   * @memberof Popper.Utils
   * @argument {Element} element
   * @argument {Element} parent
   * @argument {String} side `top` or `left`
   * @returns {Number} amount of scrolled pixels
   */
  function getTotalScroll(element, parent) {
    var side = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'top';

    var scrollParent = getScrollParent(element);
    var scroll = 0;
    var isParentFixed = isFixed(parent);

    // NOTE: I'm not sure the second line of this check is completely correct, review it if
    // someone complains about viewport problems in future
    if (isOffsetContainer(scrollParent.nodeName === 'BODY' ? window.document.documentElement : scrollParent) && (parent.contains(scrollParent) && isParentFixed || !isParentFixed)) {
      scroll = getScroll(scrollParent, side);
    }

    if (parent === scrollParent || ['BODY', 'HTML'].indexOf(scrollParent.nodeName) === -1) {
      return scroll + getTotalScroll(getParentNode(scrollParent), parent, side);
    }
    return scroll;
  }

  /**
   * Check if the given variable is a function
   * @method
   * @memberof Popper.Utils
   * @argument {*} functionToCheck - variable to check
   * @returns {Boolean} answer to: is a function?
   */
  function isFunction(functionToCheck) {
    var getType = {};
    return functionToCheck && getType.toString.call(functionToCheck) === '[object Function]';
  }

  /**
   * Helper used to know if the given modifier is enabled.
   * @method
   * @memberof Popper.Utils
   * @returns {Boolean}
   */
  function isModifierEnabled(modifiers, modifierName) {
    return modifiers.some(function (_ref) {
      var name = _ref.name,
          enabled = _ref.enabled;
      return enabled && name === modifierName;
    });
  }

  /**
   * Helper used to know if the given modifier depends from another one.
   * It checks if the needed modifier is listed and enabled.
   * @method
   * @memberof Popper.Utils
   * @param {Array} modifiers - list of modifiers
   * @param {String} requestingName - name of requesting modifier
   * @param {String} requestedName - name of requested modifier
   * @returns {Boolean}
   */
  function isModifierRequired(modifiers, requestingName, requestedName) {
    var requesting = find(modifiers, function (_ref2) {
      var name = _ref2.name;
      return name === requestingName;
    });

    return !!requesting && modifiers.some(function (modifier) {
      return modifier.name === requestedName && modifier.enabled && modifier.order < requesting.order;
    });
  }

  /**
   * Tells if a given input is a number
   * @method
   * @memberof Popper.Utils
   * @param {*} input to check
   * @return {Boolean}
   */
  function isNumeric(n) {
    return n !== '' && !isNaN(parseFloat(n)) && isFinite(n);
  }

  /**
   * Check if the given element has transforms applied to itself or a parent
   * @method
   * @memberof Popper.Utils
   * @param  {Element} element
   * @return {Boolean} answer to "isTransformed?"
   */
  function isTransformed(element) {
    if (element.nodeName === 'BODY') {
      return false;
    }
    if (getStyleComputedProperty(element, 'transform') !== 'none') {
      return true;
    }
    return getParentNode(element) ? isTransformed(getParentNode(element)) : element;
  }

  /**
   * Remove event listeners used to update the popper position
   * @method
   * @memberof Popper.Utils
   * @private
   */
  function removeEventListeners(reference, state) {
    // Remove resize event listener on window
    window.removeEventListener('resize', state.updateBound);

    // Remove scroll event listener on scroll parents
    state.scrollParents.forEach(function (target) {
      target.removeEventListener('scroll', state.updateBound);
    });

    // Reset state
    state.updateBound = null;
    state.scrollParents = [];
    state.scrollElement = null;
    state.eventsEnabled = false;
    return state;
  }

  /**
   * Loop trough the list of modifiers and run them in order, each of them will then edit the data object
   * @method
   * @memberof Popper.Utils
   * @param {Object} data
   * @param {Array} modifiers
   * @param {Function} ends
   */
  function runModifiers(modifiers, data, ends) {
    var modifiersToRun = ends === undefined ? modifiers : modifiers.slice(0, findIndex(modifiers, 'name', ends));

    modifiersToRun.forEach(function (modifier) {
      if (modifier.enabled && isFunction(modifier.function)) {
        data = modifier.function(data, modifier);
      }
    });

    return data;
  }

  /**
   * Set the attributes to the given popper
   * @method
   * @memberof Popper.Utils
   * @argument {Element} element - Element to apply the attributes to
   * @argument {Object} styles - Object with a list of properties and values which will be applied to the element
   */
  function setAttributes(element, attributes) {
    Object.keys(attributes).forEach(function (prop) {
      var value = attributes[prop];
      if (value !== false) {
        element.setAttribute(prop, attributes[prop]);
      } else {
        element.removeAttribute(prop);
      }
    });
  }

  /**
   * Set the style to the given popper
   * @method
   * @memberof Popper.Utils
   * @argument {Element} element - Element to apply the style to
   * @argument {Object} styles - Object with a list of properties and values which will be applied to the element
   */
  function setStyles(element, styles) {
    Object.keys(styles).forEach(function (prop) {
      var unit = '';
      // add unit if the value is numeric and is one of the following
      if (['width', 'height', 'top', 'right', 'bottom', 'left'].indexOf(prop) !== -1 && isNumeric(styles[prop])) {
        unit = 'px';
      }
      element.style[prop] = styles[prop] + unit;
    });
  }

  function attachToScrollParents(scrollParent, event, callback, scrollParents) {
    var isBody = scrollParent.nodeName === 'BODY';
    var target = isBody ? window : scrollParent;
    target.addEventListener(event, callback, { passive: true });

    if (!isBody) {
      attachToScrollParents(getScrollParent(target.parentNode), event, callback, scrollParents);
    }
    scrollParents.push(target);
  }

  /**
   * Setup needed event listeners used to update the popper position
   * @method
   * @memberof Popper.Utils
   * @private
   */
  function setupEventListeners(reference, options, state, updateBound) {
    // Resize event listener on window
    state.updateBound = updateBound;
    window.addEventListener('resize', state.updateBound, { passive: true });

    // Scroll event listener on scroll parents
    var scrollElement = getScrollParent(reference);
    attachToScrollParents(scrollElement, 'scroll', state.updateBound, state.scrollParents);
    state.scrollElement = scrollElement;
    state.eventsEnabled = true;

    return state;
  }

  /** @namespace Popper.Utils */
  var Utils = {
    computeAutoPlacement: computeAutoPlacement,
    debounce: debounce,
    findIndex: findIndex,
    getBordersSize: getBordersSize,
    getBoundaries: getBoundaries,
    getBoundingClientRect: getBoundingClientRect,
    getClientRect: getClientRect,
    getOffsetParent: getOffsetParent,
    getOffsetRect: getOffsetRect,
    getOffsetRectRelativeToArbitraryNode: getOffsetRectRelativeToArbitraryNode,
    getOuterSizes: getOuterSizes,
    getParentNode: getParentNode,
    getPopperOffsets: getPopperOffsets,
    getPosition: getPosition,
    getReferenceOffsets: getReferenceOffsets,
    getScroll: getScroll,
    getScrollParent: getScrollParent,
    getStyleComputedProperty: getStyleComputedProperty,
    getSupportedPropertyName: getSupportedPropertyName,
    getTotalScroll: getTotalScroll,
    getWindowSizes: getWindowSizes,
    isFixed: isFixed,
    isFunction: isFunction,
    isModifierEnabled: isModifierEnabled,
    isModifierRequired: isModifierRequired,
    isNative: isNative,
    isNumeric: isNumeric,
    isTransformed: isTransformed,
    removeEventListeners: removeEventListeners,
    runModifiers: runModifiers,
    setAttributes: setAttributes,
    setStyles: setStyles,
    setupEventListeners: setupEventListeners
  };

  /**
   * Apply the computed styles to the popper element
   * @method
   * @memberof Modifiers
   * @argument {Object} data - The data object generated by `update` method
   * @argument {Object} data.styles - List of style properties - values to apply to popper element
   * @argument {Object} data.attributes - List of attribute properties - values to apply to popper element
   * @argument {Object} options - Modifiers configuration and options
   * @returns {Object} The same data object
   */
  function applyStyle(data, options) {
    // apply the final offsets to the popper
    // NOTE: 1 DOM access here
    var styles = {
      position: data.offsets.popper.position
    };

    var attributes = {
      'x-placement': data.placement
    };

    // round top and left to avoid blurry text
    var left = Math.round(data.offsets.popper.left);
    var top = Math.round(data.offsets.popper.top);

    // if gpuAcceleration is set to true and transform is supported,
    //  we use `translate3d` to apply the position to the popper we
    // automatically use the supported prefixed version if needed
    var prefixedProperty = getSupportedPropertyName('transform');
    if (options.gpuAcceleration && prefixedProperty) {
      styles[prefixedProperty] = 'translate3d(' + left + 'px, ' + top + 'px, 0)';
      styles.top = 0;
      styles.left = 0;
      styles.willChange = 'transform';
    } else {
      // othwerise, we use the standard `left` and `top` properties
      styles.left = left;
      styles.top = top;
      styles.willChange = 'top, left';
    }

    // any property present in `data.styles` will be applied to the popper,
    // in this way we can make the 3rd party modifiers add custom styles to it
    // Be aware, modifiers could override the properties defined in the previous
    // lines of this modifier!
    setStyles(data.instance.popper, _extends({}, styles, data.styles));

    // any property present in `data.attributes` will be applied to the popper,
    // they will be set as HTML attributes of the element
    setAttributes(data.instance.popper, _extends({}, attributes, data.attributes));

    // if the arrow style has been computed, apply the arrow style
    if (data.offsets.arrow) {
      setStyles(data.arrowElement, data.offsets.arrow);
    }

    return data;
  }

  /**
   * Set the x-placement attribute before everything else because it could be used to add margins to the popper
   * margins needs to be calculated to get the correct popper offsets
   * @method
   * @memberof Popper.modifiers
   * @param {HTMLElement} reference - The reference element used to position the popper
   * @param {HTMLElement} popper - The HTML element used as popper.
   * @param {Object} options - Popper.js options
   */
  function applyStyleOnLoad(reference, popper, options, modifierOptions, state) {
    // compute reference element offsets
    var referenceOffsets = getReferenceOffsets(state, popper, reference);

    // compute auto placement, store placement inside the data object,
    // modifiers will be able to edit `placement` if needed
    // and refer to originalPlacement to know the original value
    options.placement = computeAutoPlacement(options.placement, referenceOffsets, popper, reference);

    popper.setAttribute('x-placement', options.placement);
    return options;
  }

  /**
   * Modifier used to move the arrowElements on the edge of the popper to make sure them are always between the popper and the reference element
   * It will use the CSS outer size of the arrowElement element to know how many pixels of conjuction are needed
   * @method
   * @memberof Modifiers
   * @argument {Object} data - The data object generated by update method
   * @argument {Object} options - Modifiers configuration and options
   * @returns {Object} The data object, properly modified
   */
  function arrow(data, options) {
    // arrow depends on keepTogether in order to work
    if (!isModifierRequired(data.instance.modifiers, 'arrow', 'keepTogether')) {
      console.warn('WARNING: `keepTogether` modifier is required by arrow modifier in order to work, be sure to include it before `arrow`!');
      return data;
    }

    var arrowElement = options.element;

    // if arrowElement is a string, suppose it's a CSS selector
    if (typeof arrowElement === 'string') {
      arrowElement = data.instance.popper.querySelector(arrowElement);

      // if arrowElement is not found, don't run the modifier
      if (!arrowElement) {
        return data;
      }
    } else {
      // if the arrowElement isn't a query selector we must check that the
      // provided DOM node is child of its popper node
      if (!data.instance.popper.contains(arrowElement)) {
        console.warn('WARNING: `arrow.element` must be child of its popper element!');
        return data;
      }
    }

    var placement = data.placement.split('-')[0];
    var popper = getClientRect(data.offsets.popper);
    var reference = data.offsets.reference;
    var isVertical = ['left', 'right'].indexOf(placement) !== -1;

    var len = isVertical ? 'height' : 'width';
    var side = isVertical ? 'top' : 'left';
    var altSide = isVertical ? 'left' : 'top';
    var opSide = isVertical ? 'bottom' : 'right';
    var arrowElementSize = getOuterSizes(arrowElement)[len];

    //
    // extends keepTogether behavior making sure the popper and its reference have enough pixels in conjuction
    //

    // top/left side
    if (reference[opSide] - arrowElementSize < popper[side]) {
      data.offsets.popper[side] -= popper[side] - (reference[opSide] - arrowElementSize);
    }
    // bottom/right side
    if (reference[side] + arrowElementSize > popper[opSide]) {
      data.offsets.popper[side] += reference[side] + arrowElementSize - popper[opSide];
    }

    // compute center of the popper
    var center = reference[side] + reference[len] / 2 - arrowElementSize / 2;

    // Compute the sideValue using the updated popper offsets
    var sideValue = center - getClientRect(data.offsets.popper)[side];

    // prevent arrowElement from being placed not contiguously to its popper
    sideValue = Math.max(Math.min(popper[len] - arrowElementSize, sideValue), 0);

    data.arrowElement = arrowElement;
    data.offsets.arrow = {};
    data.offsets.arrow[side] = sideValue;
    data.offsets.arrow[altSide] = ''; // make sure to unset any eventual altSide value from the DOM node

    return data;
  }

  /**
   * Get the opposite placement variation of the given one/
   * @method
   * @memberof Popper.Utils
   * @argument {String} placement variation
   * @returns {String} flipped placement variation
   */
  function getOppositeVariation(variation) {
    if (variation === 'end') {
      return 'start';
    } else if (variation === 'start') {
      return 'end';
    }
    return variation;
  }

  /**
   * Modifier used to flip the placement of the popper when the latter is starting overlapping its reference element.
   * Requires the `preventOverflow` modifier before it in order to work.
   * **NOTE:** data.instance modifier will run all its previous modifiers everytime it tries to flip the popper!
   * @method
   * @memberof Modifiers
   * @argument {Object} data - The data object generated by update method
   * @argument {Object} options - Modifiers configuration and options
   * @returns {Object} The data object, properly modified
   */
  function flip(data, options) {
    // if `inner` modifier is enabled, we can't use the `flip` modifier
    if (isModifierEnabled(data.instance.modifiers, 'inner')) {
      return data;
    }

    if (data.flipped && data.placement === data.originalPlacement) {
      // seems like flip is trying to loop, probably there's not enough space on any of the flippable sides
      return data;
    }

    var boundaries = getBoundaries(data.instance.popper, data.instance.reference, options.padding, options.boundariesElement);

    var placement = data.placement.split('-')[0];
    var placementOpposite = getOppositePlacement(placement);
    var variation = data.placement.split('-')[1] || '';

    var flipOrder = [];

    if (options.behavior === 'flip') {
      flipOrder = [placement, placementOpposite];
    } else {
      flipOrder = options.behavior;
    }

    flipOrder.forEach(function (step, index) {
      if (placement !== step || flipOrder.length === index + 1) {
        return data;
      }

      placement = data.placement.split('-')[0];
      placementOpposite = getOppositePlacement(placement);

      var popperOffsets = getClientRect(data.offsets.popper);
      var refOffsets = data.offsets.reference;

      // using floor because the reference offsets may contain decimals we are not going to consider here
      var floor = Math.floor;
      var overlapsRef = placement === 'left' && floor(popperOffsets.right) > floor(refOffsets.left) || placement === 'right' && floor(popperOffsets.left) < floor(refOffsets.right) || placement === 'top' && floor(popperOffsets.bottom) > floor(refOffsets.top) || placement === 'bottom' && floor(popperOffsets.top) < floor(refOffsets.bottom);

      var overflowsLeft = floor(popperOffsets.left) < floor(boundaries.left);
      var overflowsRight = floor(popperOffsets.right) > floor(boundaries.right);
      var overflowsTop = floor(popperOffsets.top) < floor(boundaries.top);
      var overflowsBottom = floor(popperOffsets.bottom) > floor(boundaries.bottom);

      var overflowsBoundaries = placement === 'left' && overflowsLeft || placement === 'right' && overflowsRight || placement === 'top' && overflowsTop || placement === 'bottom' && overflowsBottom;

      // flip the variation if required
      var isVertical = ['top', 'bottom'].indexOf(placement) !== -1;
      var flippedVariation = !!options.flipVariations && (isVertical && variation === 'start' && overflowsLeft || isVertical && variation === 'end' && overflowsRight || !isVertical && variation === 'start' && overflowsTop || !isVertical && variation === 'end' && overflowsBottom);

      if (overlapsRef || overflowsBoundaries || flippedVariation) {
        // this boolean to detect any flip loop
        data.flipped = true;

        if (overlapsRef || overflowsBoundaries) {
          placement = flipOrder[index + 1];
        }

        if (flippedVariation) {
          variation = getOppositeVariation(variation);
        }

        data.placement = placement + (variation ? '-' + variation : '');
        data.offsets.popper = getPopperOffsets(data.instance.state.position, data.instance.popper, data.offsets.reference, data.placement);

        data = runModifiers(data.instance.modifiers, data, 'flip');
      }
    });
    return data;
  }

  /**
   * Modifier used to make sure the popper is always near its reference element
   * It cares only about the first axis, you can still have poppers with margin
   * between the popper and its reference element.
   * @method
   * @memberof Modifiers
   * @argument {Object} data - The data object generated by update method
   * @argument {Object} options - Modifiers configuration and options
   * @returns {Object} The data object, properly modified
   */
  function keepTogether(data) {
    var popper = getClientRect(data.offsets.popper);
    var reference = data.offsets.reference;
    var placement = data.placement.split('-')[0];
    var floor = Math.floor;
    var isVertical = ['top', 'bottom'].indexOf(placement) !== -1;
    var side = isVertical ? 'right' : 'bottom';
    var opSide = isVertical ? 'left' : 'top';
    var measurement = isVertical ? 'width' : 'height';

    if (popper[side] < floor(reference[opSide])) {
      data.offsets.popper[opSide] = floor(reference[opSide]) - popper[measurement];
    }
    if (popper[opSide] > floor(reference[side])) {
      data.offsets.popper[opSide] = floor(reference[side]);
    }

    return data;
  }

  /**
   * Modifier used to add an offset to the popper, useful if you more granularity positioning your popper.
   * The offsets will shift the popper on the side of its reference element.
   * @method
   * @memberof Modifiers
   * @argument {Object} data - The data object generated by update method
   * @argument {Object} options - Modifiers configuration and options
   * @argument {Number|String} options.offset=0
   *      Basic usage allows a number used to nudge the popper by the given amount of pixels.
   *      You can pass a percentage value as string (eg. `20%`) to nudge by the given percentage (relative to reference element size)
   *      Other supported units are `vh` and `vw` (relative to viewport)
   *      Additionally, you can pass a pair of values (eg. `10 20` or `2vh 20%`) to nudge the popper
   *      on both axis.
   *      A note about percentage values, if you want to refer a percentage to the popper size instead of the reference element size,
   *      use `%p` instead of `%` (eg: `20%p`). To make it clearer, you can replace `%` with `%r` and use eg.`10%p 25%r`.
   *      > **Heads up!** The order of the axis is relative to the popper placement: `bottom` or `top` are `X,Y`, the other are `Y,X`
   * @returns {Object} The data object, properly modified
   */
  function offset(data, options) {
    var placement = data.placement;
    var popper = data.offsets.popper;

    var offsets = void 0;
    if (isNumeric(options.offset)) {
      offsets = [options.offset, 0];
    } else {
      // split the offset in case we are providing a pair of offsets separated
      // by a blank space
      offsets = options.offset.split(' ');

      // itherate through each offset to compute them in case they are percentages
      offsets = offsets.map(function (offset, index) {
        // separate value from unit
        var split = offset.match(/(\d*\.?\d*)(.*)/);
        var value = +split[1];
        var unit = split[2];

        // use height if placement is left or right and index is 0 otherwise use width
        // in this way the first offset will use an axis and the second one
        // will use the other one
        var useHeight = placement.indexOf('right') !== -1 || placement.indexOf('left') !== -1;

        if (index === 1) {
          useHeight = !useHeight;
        }

        var measurement = useHeight ? 'height' : 'width';

        // if is a percentage relative to the popper (%p), we calculate the value of it using
        // as base the sizes of the popper
        // if is a percentage (% or %r), we calculate the value of it using as base the
        // sizes of the reference element
        if (unit.indexOf('%') === 0) {
          var element = void 0;
          switch (unit) {
            case '%p':
              element = data.offsets.popper;
              break;
            case '%':
            case '$r':
            default:
              element = data.offsets.reference;
          }

          var rect = getClientRect(element);
          var len = rect[measurement];
          return len / 100 * value;
        } else if (unit === 'vh' || unit === 'vw') {
          // if is a vh or vw, we calculate the size based on the viewport
          var size = void 0;
          if (unit === 'vh') {
            size = Math.max(document.documentElement.clientHeight, window.innerHeight || 0);
          } else {
            size = Math.max(document.documentElement.clientWidth, window.innerWidth || 0);
          }
          return size / 100 * value;
        } else if (unit === 'px') {
          // if is an explicit pixel unit, we get rid of the unit and keep the value
          return +value;
        } else {
          // if is an implicit unit, it's px, and we return just the value
          return +offset;
        }
      });
    }

    if (data.placement.indexOf('left') !== -1) {
      popper.top += offsets[0];
      popper.left -= offsets[1] || 0;
    } else if (data.placement.indexOf('right') !== -1) {
      popper.top += offsets[0];
      popper.left += offsets[1] || 0;
    } else if (data.placement.indexOf('top') !== -1) {
      popper.left += offsets[0];
      popper.top -= offsets[1] || 0;
    } else if (data.placement.indexOf('bottom') !== -1) {
      popper.left += offsets[0];
      popper.top += offsets[1] || 0;
    }
    return data;
  }

  /**
   * Modifier used to prevent the popper from being positioned outside the boundary.
   *
   * An scenario exists where the reference itself is not within the boundaries. We can
   * say it has "escaped the boundaries" — or just "escaped". In this case we need to
   * decide whether the popper should either:
   *
   * - detach from the reference and remain "trapped" in the boundaries, or
   * - if it should be ignore the boundary and "escape with the reference"
   *
   * When `escapeWithReference` is `true`, and reference is completely outside the
   * boundaries, the popper will overflow (or completely leave) the boundaries in order
   * to remain attached to the edge of the reference.
   *
   * @method
   * @memberof Modifiers
   * @argument {Object} data - The data object generated by `update` method
   * @argument {Object} options - Modifiers configuration and options
   * @returns {Object} The data object, properly modified
   */
  function preventOverflow(data, options) {
    var boundariesElement = options.boundariesElement || getOffsetParent(data.instance.popper);
    var boundaries = getBoundaries(data.instance.popper, data.instance.reference, options.padding, boundariesElement);
    options.boundaries = boundaries;

    var order = options.priority;
    var popper = getClientRect(data.offsets.popper);

    var check = {
      primary: function primary(placement) {
        var _ref3;

        var value = popper[placement];
        if (popper[placement] < boundaries[placement] && !options.escapeWithReference) {
          value = Math.max(popper[placement], boundaries[placement]);
        }
        return _ref3 = {}, _ref3[placement] = value, _ref3;
      },
      secondary: function secondary(placement) {
        var _ref4;

        var mainSide = placement === 'right' ? 'left' : 'top';
        var value = popper[mainSide];
        if (popper[placement] > boundaries[placement] && !options.escapeWithReference) {
          value = Math.min(popper[mainSide], boundaries[placement] - (placement === 'right' ? popper.width : popper.height));
        }
        return _ref4 = {}, _ref4[mainSide] = value, _ref4;
      }
    };

    order.forEach(function (placement) {
      var side = ['left', 'top'].indexOf(placement) !== -1 ? 'primary' : 'secondary';
      popper = _extends({}, popper, check[side](placement));
    });

    data.offsets.popper = popper;

    return data;
  }

  /**
   * Modifier used to shift the popper on the start or end of its reference element side
   * @method
   * @memberof Modifiers
   * @argument {Object} data - The data object generated by `update` method
   * @argument {Object} options - Modifiers configuration and options
   * @returns {Object} The data object, properly modified
   */
  function shift(data) {
    var placement = data.placement;
    var basePlacement = placement.split('-')[0];
    var shiftvariation = placement.split('-')[1];

    // if shift shiftvariation is specified, run the modifier
    if (shiftvariation) {
      var _start, _end;

      var reference = data.offsets.reference;
      var popper = getClientRect(data.offsets.popper);
      var isVertical = ['bottom', 'top'].indexOf(basePlacement) !== -1;
      var side = isVertical ? 'left' : 'top';
      var measurement = isVertical ? 'width' : 'height';

      var shiftOffsets = {
        start: (_start = {}, _start[side] = reference[side], _start),
        end: (_end = {}, _end[side] = reference[side] + reference[measurement] - popper[measurement], _end)
      };

      data.offsets.popper = _extends({}, popper, shiftOffsets[shiftvariation]);
    }

    return data;
  }

  /**
   * Modifier used to hide the popper when its reference element is outside of the
   * popper boundaries. It will set an x-hidden attribute which can be used to hide
   * the popper when its reference is out of boundaries.
   * @method
   * @memberof Modifiers
   * @argument {Object} data - The data object generated by update method
   * @argument {Object} options - Modifiers configuration and options
   * @returns {Object} The data object, properly modified
   */
  function hide(data) {
    if (!isModifierRequired(data.instance.modifiers, 'hide', 'preventOverflow')) {
      console.warn('WARNING: preventOverflow modifier is required by hide modifier in order to work, be sure to include it before hide!');
      return data;
    }

    var refRect = data.offsets.reference;
    var bound = find(data.instance.modifiers, function (modifier) {
      return modifier.name === 'preventOverflow';
    }).boundaries;

    if (refRect.bottom < bound.top || refRect.left > bound.right || refRect.top > bound.bottom || refRect.right < bound.left) {
      // Avoid unnecessary DOM access if visibility hasn't changed
      if (data.hide === true) {
        return data;
      }

      data.hide = true;
      data.attributes['x-out-of-boundaries'] = '';
    } else {
      // Avoid unnecessary DOM access if visibility hasn't changed
      if (data.hide === false) {
        return data;
      }

      data.hide = false;
      data.attributes['x-out-of-boundaries'] = false;
    }

    return data;
  }

  /**
   * Modifier used to make the popper flow toward the inner of the reference element.
   * By default, when this modifier is disabled, the popper will be placed outside
   * the reference element.
   * @method
   * @memberof Modifiers
   * @argument {Object} data - The data object generated by `update` method
   * @argument {Object} options - Modifiers configuration and options
   * @returns {Object} The data object, properly modified
   */
  function inner(data) {
    var placement = data.placement;
    var basePlacement = placement.split('-')[0];
    var popper = getClientRect(data.offsets.popper);
    var reference = getClientRect(data.offsets.reference);
    var isHoriz = ['left', 'right'].indexOf(basePlacement) !== -1;

    var subtractLength = ['top', 'left'].indexOf(basePlacement) === -1;

    popper[isHoriz ? 'left' : 'top'] = reference[placement] - (subtractLength ? popper[isHoriz ? 'width' : 'height'] : 0);

    data.placement = getOppositePlacement(placement);
    data.offsets.popper = getClientRect(popper);

    return data;
  }

  /**
   * Modifiers are plugins used to alter the behavior of your poppers.
   * Popper.js uses a set of 7 modifiers to provide all the basic functionalities
   * needed by the library.
   *
   * Each modifier is an object containing several properties listed below.
   * @namespace Modifiers
   * @param {Object} modifier - Modifier descriptor
   * @param {Integer} modifier.order
   *      The `order` property defines the execution order of the modifiers.
   *      The built-in modifiers have orders with a gap of 100 units in between,
   *      this allows you to inject additional modifiers between the existing ones
   *      without having to redefine the order of all of them.
   *      The modifiers are executed starting from the one with the lowest order.
   * @param {Boolean} modifier.enabled - When `true`, the modifier will be used.
   * @param {Modifiers~modifier} modifier.function - Modifier function.
   * @param {Modifiers~onLoad} modifier.onLoad - Function executed on popper initalization
   * @return {Object} data - Each modifier must return the modified `data` object.
   */
  var modifiers = {
    shift: {
      order: 100,
      enabled: true,
      function: shift
    },
    offset: {
      order: 200,
      enabled: true,
      function: offset,
      // nudges popper from its origin by the given amount of pixels (can be negative)
      offset: 0
    },
    preventOverflow: {
      order: 300,
      enabled: true,
      function: preventOverflow,
      // popper will try to prevent overflow following these priorities
      //  by default, then, it could overflow on the left and on top of the boundariesElement
      priority: ['left', 'right', 'top', 'bottom'],
      // amount of pixel used to define a minimum distance between the boundaries and the popper
      // this makes sure the popper has always a little padding between the edges of its container
      padding: 5,
      boundariesElement: 'scrollParent'
    },
    keepTogether: {
      order: 400,
      enabled: true,
      function: keepTogether
    },
    arrow: {
      order: 500,
      enabled: true,
      function: arrow,
      // selector or node used as arrow
      element: '[x-arrow]'
    },
    flip: {
      order: 600,
      enabled: true,
      function: flip,
      // the behavior used to change the popper's placement
      behavior: 'flip',
      // the popper will flip if it hits the edges of the boundariesElement - padding
      padding: 5,
      boundariesElement: 'viewport'
    },
    inner: {
      order: 700,
      enabled: false,
      function: inner
    },
    hide: {
      order: 800,
      enabled: true,
      function: hide
    },
    applyStyle: {
      order: 900,
      enabled: true,
      // if true, it uses the CSS 3d transformation to position the popper
      gpuAcceleration: true,
      function: applyStyle,
      onLoad: applyStyleOnLoad
    }
  };

  /**
    * Modifiers can edit the `data` object to change the beheavior of the popper.
    * This object contains all the informations used by Popper.js to compute the
    * popper position.
    * The modifier can edit the data as needed, and then `return` it as result.
    *
    * @callback Modifiers~modifier
    * @param {dataObject} data
    * @return {dataObject} modified data
    */

  /**
    * The `dataObject` is an object containing all the informations used by Popper.js
    * this object get passed to modifiers and to the `onCreate` and `onUpdate` callbacks.
    * @name dataObject
    * @property {Object} data.instance The Popper.js instance
    * @property {String} data.placement Placement applied to popper
    * @property {String} data.originalPlacement Placement originally defined on init
    * @property {Boolean} data.flipped True if popper has been flipped by flip modifier
    * @property {Boolean} data.hide True if the reference element is out of boundaries, useful to know when to hide the popper.
    * @property {HTMLElement} data.arrowElement Node used as arrow by arrow modifier
    * @property {Object} data.styles Any CSS property defined here will be applied to the popper, it expects the JavaScript nomenclature (eg. `marginBottom`)
    * @property {Object} data.boundaries Offsets of the popper boundaries
    * @property {Object} data.offsets The measurements of popper, reference and arrow elements.
    * @property {Object} data.offsets.popper `top`, `left`, `width`, `height` values
    * @property {Object} data.offsets.reference `top`, `left`, `width`, `height` values
    * @property {Object} data.offsets.arro] `top` and `left` offsets, only one of them will be different from 0
    */

  // Utils
  // Modifiers
  // default options
  var DEFAULTS = {
    // placement of the popper
    placement: 'bottom',

    // whether events (resize, scroll) are initially enabled
    eventsEnabled: true,

    /**
       * Callback called when the popper is created.
       * By default, is set to no-op.
       * Access Popper.js instance with `data.instance`.
       * @callback createCallback
       * @static
       * @param {dataObject} data
       */
    onCreate: function onCreate() {},

    /**
       * Callback called when the popper is updated, this callback is not called
       * on the initialization/creation of the popper, but only on subsequent
       * updates.
       * By default, is set to no-op.
       * Access Popper.js instance with `data.instance`.
       * @callback updateCallback
       * @static
       * @param {dataObject} data
       */
    onUpdate: function onUpdate() {},

    // list of functions used to modify the offsets before they are applied to the popper
    modifiers: modifiers
  };

  /**
   * Create a new Popper.js instance
   * @class Popper
   * @param {HTMLElement} reference - The reference element used to position the popper
   * @param {HTMLElement} popper - The HTML element used as popper.
   * @param {Object} options
   * @param {String} options.placement=bottom
   *      Placement of the popper accepted values: `top(-start, -end), right(-start, -end), bottom(-start, -end),
   *      left(-start, -end)`
   *
   * @param {Boolean} options.eventsEnabled=true
   *      Whether events (resize, scroll) are initially enabled
   * @param {Boolean} options.gpuAcceleration=true
   *      When this property is set to true, the popper position will be applied using CSS3 translate3d, allowing the
   *      browser to use the GPU to accelerate the rendering.
   *      If set to false, the popper will be placed using `top` and `left` properties, not using the GPU.
   *
   * @param {Boolean} options.removeOnDestroy=false
   *      Set to true if you want to automatically remove the popper when you call the `destroy` method.
   *
   * @param {Object} options.modifiers
   *      List of functions used to modify the data before they are applied to the popper (see source code for default values)
   *
   * @param {Object} options.modifiers.arrow - Arrow modifier configuration
   * @param {String|HTMLElement} options.modifiers.arrow.element='[x-arrow]'
   *      The DOM Node used as arrow for the popper, or a CSS selector used to get the DOM node. It must be child of
   *      its parent Popper. Popper.js will apply to the given element the style required to align the arrow with its
   *      reference element.
   *      By default, it will look for a child node of the popper with the `x-arrow` attribute.
   *
   * @param {Object} options.modifiers.offset - Offset modifier configuration
   * @param {Number} options.modifiers.offset.offset=0
   *      Amount of pixels the popper will be shifted (can be negative).
   *
   * @param {Object} options.modifiers.preventOverflow - PreventOverflow modifier configuration
   * @param {Array} [options.modifiers.preventOverflow.priority=['left', 'right', 'top', 'bottom']]
   *      Priority used when Popper.js tries to avoid overflows from the boundaries, they will be checked in order,
   *      this means that the last one will never overflow
   * @param {String|HTMLElement} options.modifiers.preventOverflow.boundariesElement='scrollParent'
   *      Boundaries used by the modifier, can be `scrollParent`, `window`, `viewport` or any DOM element.
   * @param {Number} options.modifiers.preventOverflow.padding=5
   *      Amount of pixels used to define a minimum distance between the boundaries and the popper
   *      this makes sure the popper has always a little padding between the edges of its container.
   *
   * @param {Object} options.modifiers.flip - Flip modifier configuration
   * @param {String|Array} options.modifiers.flip.behavior='flip'
   *      The behavior used by the `flip` modifier to change the placement of the popper when the latter is trying to
   *      overlap its reference element. Defining `flip` as value, the placement will be flipped on
   *      its axis (`right - left`, `top - bottom`).
   *      You can even pass an array of placements (eg: `['right', 'left', 'top']` ) to manually specify
   *      how alter the placement when a flip is needed. (eg. in the above example, it would first flip from right to left,
   *      then, if even in its new placement, the popper is overlapping its reference element, it will be moved to top)
   * @param {String|HTMLElement} options.modifiers.flip.boundariesElement='viewport'
   *      The element which will define the boundaries of the popper position, the popper will never be placed outside
   *      of the defined boundaries (except if `keepTogether` is enabled)
   *
   * @param {Object} options.modifiers.inner - Inner modifier configuration
   * @param {Number} options.modifiers.inner.enabled=false
   *      Set to `true` to make the popper flow toward the inner of the reference element.
   *
   * @param {Number} options.modifiers.flip.padding=5
   *      Amount of pixels used to define a minimum distance between the boundaries and the popper
   *      this makes sure the popper will flip before it touches the edge of the boundaries,
   *      making it have always a little padding between the edges of its container.
   *
   * @param {createCallback} options.onCreate - onCreate callback
   *      Function called after the Popper has been instantiated.
   *
   * @param {updateCallback} options.onUpdate - onUpdate callback
   *      Function called on subsequent updates of Popper.
   *
   * @return {Object} instance - The generated Popper.js instance
   */

  var Popper = function () {
    function Popper(reference, popper) {
      var _this = this;

      var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

      _classCallCheck(this, Popper);

      this.scheduleUpdate = function () {
        return requestAnimationFrame(_this.update);
      };

      // make update() debounced, so that it only runs at most once-per-tick
      this.update = debounce(this.update.bind(this));

      // with {} we create a new object with the options inside it
      this.options = _extends({}, Popper.Defaults, options);

      // init state
      this.state = {
        isDestroyed: false,
        isCreated: false,
        scrollParents: []
      };

      // get reference and popper elements (allow jQuery wrappers)
      this.reference = reference.jquery ? reference[0] : reference;
      this.popper = popper.jquery ? popper[0] : popper;

      // make sure to apply the popper position before any computation
      setStyles(this.popper, { position: 'absolute' });

      // refactoring modifiers' list (Object => Array)
      this.modifiers = Object.keys(Popper.Defaults.modifiers).map(function (name) {
        return _extends({
          name: name
        }, Popper.Defaults.modifiers[name]);
      });

      // assign default values to modifiers, making sure to override them with
      // the ones defined by user
      this.modifiers = this.modifiers.map(function (defaultConfig) {
        var userConfig = options.modifiers && options.modifiers[defaultConfig.name] || {};
        return _extends({}, defaultConfig, userConfig);
      });

      // add custom modifiers to the modifiers list
      if (options.modifiers) {
        this.options.modifiers = _extends({}, Popper.Defaults.modifiers, options.modifiers);
        Object.keys(options.modifiers).forEach(function (name) {
          // take in account only custom modifiers
          if (Popper.Defaults.modifiers[name] === undefined) {
            var modifier = options.modifiers[name];
            modifier.name = name;
            _this.modifiers.push(modifier);
          }
        });
      }

      // sort the modifiers by order
      this.modifiers = this.modifiers.sort(function (a, b) {
        return a.order - b.order;
      });

      // modifiers have the ability to execute arbitrary code when Popper.js get inited
      // such code is executed in the same order of its modifier
      // they could add new properties to their options configuration
      // BE AWARE: don't add options to `options.modifiers.name` but to `modifierOptions`!
      this.modifiers.forEach(function (modifierOptions) {
        if (modifierOptions.enabled && isFunction(modifierOptions.onLoad)) {
          modifierOptions.onLoad(_this.reference, _this.popper, _this.options, modifierOptions, _this.state);
        }
      });

      // determine how we should set the origin of offsets
      this.state.isParentTransformed = isTransformed(this.popper.parentNode);

      // fire the first update to position the popper in the right place
      this.update();

      var eventsEnabled = this.options.eventsEnabled;
      if (eventsEnabled) {
        // setup event listeners, they will take care of update the position in specific situations
        this.enableEventListeners();
      }

      this.state.eventsEnabled = eventsEnabled;
    }

    //
    // Methods
    //

    /**
       * Updates the position of the popper, computing the new offsets and applying the new style
       * Prefer `scheduleUpdate` over `update` because of performance reasons
       * @method
       * @memberof Popper
       */


    Popper.prototype.update = function update() {
      // if popper is destroyed, don't perform any further update
      if (this.state.isDestroyed) {
        return;
      }

      var data = {
        instance: this,
        styles: {},
        attributes: {},
        flipped: false,
        offsets: {}
      };

      // compute reference element offsets
      data.offsets.reference = getReferenceOffsets(this.state, this.popper, this.reference);

      // compute auto placement, store placement inside the data object,
      // modifiers will be able to edit `placement` if needed
      // and refer to originalPlacement to know the original value
      data.placement = computeAutoPlacement(this.options.placement, data.offsets.reference, this.popper, this.reference);

      // store the computed placement inside `originalPlacement`
      data.originalPlacement = this.options.placement;

      // compute the popper offsets
      data.offsets.popper = getPopperOffsets(this.state, this.popper, data.offsets.reference, data.placement);

      // run the modifiers
      data = runModifiers(this.modifiers, data);

      // the first `update` will call `onCreate` callback
      // the other ones will call `onUpdate` callback
      if (!this.state.isCreated) {
        this.state.isCreated = true;
        this.options.onCreate(data);
      } else {
        this.options.onUpdate(data);
      }
    };

    /**
       * Schedule an update, it will run on the next UI update available
       * @method scheduleUpdate
       * @memberof Popper
       */

    /**
       * Destroy the popper
       * @method
       * @memberof Popper
       */


    Popper.prototype.destroy = function destroy() {
      this.state.isDestroyed = true;

      // touch DOM only if `applyStyle` modifier is enabled
      if (isModifierEnabled(this.modifiers, 'applyStyle')) {
        this.popper.removeAttribute('x-placement');
        this.popper.style.left = '';
        this.popper.style.position = '';
        this.popper.style.top = '';
        this.popper.style[getSupportedPropertyName('transform')] = '';
      }

      this.disableEventListeners();

      // remove the popper if user explicity asked for the deletion on destroy
      // do not use `remove` because IE11 doesn't support it
      if (this.options.removeOnDestroy) {
        this.popper.parentNode.removeChild(this.popper);
      }
      return this;
    };

    /**
       * it will add resize/scroll events and start recalculating
       * position of the popper element when they are triggered
       * @method
       * @memberof Popper
       */


    Popper.prototype.enableEventListeners = function enableEventListeners() {
      if (!this.state.eventsEnabled) {
        this.state = setupEventListeners(this.reference, this.options, this.state, this.scheduleUpdate);
      }
    };

    /**
       * it will remove resize/scroll events and won't recalculate
       * popper position when they are triggered. It also won't trigger onUpdate callback anymore,
       * unless you call 'update' method manually.
       * @method
       * @memberof Popper
       */


    Popper.prototype.disableEventListeners = function disableEventListeners() {
      if (this.state.eventsEnabled) {
        window.cancelAnimationFrame(this.scheduleUpdate);
        this.state = removeEventListeners(this.reference, this.state);
      }
    };

    /**
       * Collection of utilities useful when writing custom modifiers
       * @memberof Popper
       */

    /**
       * List of accepted placements to use as values of the `placement` option
       * @memberof Popper
       */

    /**
       * Default Popper.js options
       * @memberof Popper
       */


    return Popper;
  }();

  Popper.Utils = Utils;
  Popper.placements = ['auto', 'auto-start', 'auto-end', 'top', 'top-start', 'top-end', 'right', 'right-start', 'right-end', 'bottom', 'bottom-start', 'bottom-end', 'left', 'left-start', 'left-end'];
  Popper.Defaults = DEFAULTS;

  return Popper;
});

});

/**
 * @author: laurent blanes <laurent.blanes@gmail.com>
 */
var BASE_CLASS = 'vue-tooltip';

/**
 * usage:
 *
 * // basic usage:
 * <div v-tooltip="'my content'">
 * or
 * <div v-tooltip="{content: 'my content'}">
 *
 * // change position of tooltip
 * // options: bottom (default) | top | left | right
 * <div v-tooltip.top="{content: 'my content'}">
 *
 * // add custom class
 * <div v-tooltip="{class: 'custom-class', content: 'my content'}">
 *
 * // toggle visibility
 * <div v-tooltip="{visible: false, content: 'my content'}">
 */
var Tooltip$1 = {
    name: 'tooltip',
    config: {},
    install: function install(Vue) {
        Vue.directive('tooltip', {
            bind: function bind(el, binding, vnode) {
                var $popper = null;
                var placement = 'bottom';

                if (binding.modifiers.left) {
                    placement = 'left';
                } else if (binding.modifiers.right) {
                    placement = 'right';
                } else if (binding.modifiers.top) {
                    placement = 'top';
                } else if (binding.modifiers.bottom) {
                    placement = 'bottom';
                }

                // wrapper
                $popper = document.createElement('div');
                $popper.setAttribute('class', BASE_CLASS);
                popper.Utils.setStyles($popper, { visibility: 'hidden' });

                // make arrow
                var $arrow = document.createElement('div');
                $arrow.setAttribute('x-arrow', '');
                $popper.appendChild($arrow);

                // make content container
                var $content = document.createElement('div');
                $content.setAttribute('class', 'vue-tooltip-content');
                $popper.appendChild($content);

                document.querySelector('body').appendChild($popper);

                var options = Object.assign({}, binding.value, {
                    placement: placement,
                    onCreate: function onCreate(e) {
                        setProperties(el, binding);
                        setAttributes($popper, $content, el);
                    },
                    onUpdate: function onUpdate(e) {
                        setAttributes($popper, $content, el);
                    }
                });
                el.popper = new popper(el, $popper, options);
            },
            inserted: function inserted(el, binding, vnode, oldVnode) {
                el.addEventListener('mouseover', onMouseOver);
                el.addEventListener('mouseout', onMouseOut);
                el.addEventListener('mouseleave', onMouseOut);
            },
            componentUpdated: function componentUpdated(el, binding, vnode, oldVnode) {
                setProperties(el, binding);
            },
            unbind: function unbind(el, binding, vnode, oldVnode) {
                el.removeEventListener('mouseover', onMouseOver);
                el.removeEventListener('mouseout', onMouseOut);
                el.removeEventListener('mouseleave', onMouseOut);
                document.querySelector('body').removeChild(el.popper.popper);
            }
        });
    }
};

function setProperties(el, binding) {
    if (typeof binding.value === 'string') {
        el.popper._class = '';
        el.popper._content = binding.value;
        el.popper._visible = true;
    } else {
        el.popper._class = binding.value.class || '';
        el.popper._content = binding.value.content;
        el.popper._visible = binding.value.visible !== false;
    }
}

function setAttributes($popper, $content, el) {
    $content.innerHTML = el.popper._content;
    popper.Utils.setStyles(el.popper.popper, {
        display: el.popper._visible ? 'inline-block' : 'none'
    });
    var classes = BASE_CLASS + ' ' + el.popper._class;
    $popper.setAttribute('class', classes.trim());
}

function onMouseOver(e) {
    var el = e.target;
    popper.Utils.setStyles(el.popper.popper, { visibility: 'visible' });
    el.popper.update();
}

function onMouseOut(e) {
    var el = e.target;
    popper.Utils.setStyles(el.popper.popper, { visibility: 'hidden' });
}

if (typeof window !== 'undefined' && window.Vue) {
    window.Vue.use(Tooltip$1);
}

return Tooltip$1;

})));
